// A little application that takes that type name for a type
// as input (along with its containing assembly name) and
// displays an indication as to whether or not the specified
// type is serializable.
//
// Mike Woodring
// http://www.bearcanyon.com
// http://staff.develop.com/woodring
//
using System;
using System.Runtime.Serialization;

class IsSerializableApp
{
	static void Main(string[] args)
	{
        if( args.Length != 2 )
        {
            Console.WriteLine("Usage: sa assemblyName typeName");
            return;
        }

        string asmName = args[0];
        string typeName = args[1];

        object o = LoadType(asmName, typeName);
        
        if( o != null )
        {
            Type t = o.GetType();

            if( t.IsSerializable )
            {
                Console.WriteLine("{0} is serializable", t.FullName);

                if( typeof(ISerializable).IsAssignableFrom(t) )
                {
                    Console.WriteLine("  Implements ISerializable");
                }

                if( typeof(IDeserializationCallback).IsAssignableFrom(t) )
                {
                    Console.WriteLine("  Implements IDeserializationCallback");
                }
            }
            else
            {
                Console.WriteLine("{0} is not serializable", t.FullName);
            }
        }
        else
        {
            Console.WriteLine("Failed to instantiate {0}", typeName);
        }
	}

    static object LoadType( string asmName, string tn )
    {
        // Algorithm:
        //
        //  Try to load typeName
        //  Try to load "System." + typeName
        //  Give up
        //
        string[] TypeNamesToTry =
        {
            tn,
            "System." + tn
        };

        foreach( string typeName in TypeNamesToTry )
        {
            try
            {
                object o = Activator.CreateInstance(asmName, typeName).Unwrap();

                if( o != null )
                {
                    return(o);
                }
            }
            catch
            {
            }
        }

        return(null);
    }
}
